/*
This file is part of Time Tagger software defined digital data acquisition.

Copyright (C) 2011-2016 Swabian Instruments
All Rights Reserved

Unauthorized copying of this file is strictly prohibited.
*/
#pragma once

#include "TimeTagger.h"

#include <vector>

/*
*   Example to explain how to create a custom measurement class
*/
class CustomMeasurement : public IteratorBase {
public:

  // constructor
  CustomMeasurement(TimeTaggerBase* tagger, std::vector<channel_t> channels);

  ~CustomMeasurement();

  /*
  * Within getData, the internal data (customData) is copied to a newly allocated allocated memory, and the
  * pointer to this location is returned.
  */
  GET_DATA_1D(getData, long long, array_out, );

  
protected:
  /*
  * next recieves and handles the incoming tags
  */
  bool next_impl(std::vector<Tag>& incoming_tags, timestamp_t begin_time, timestamp_t end_time) override;

  /*
  * reset measurement
  */
  void clear_impl() override;

  /*
  * callback before the measurement is started
  */
  void on_start() override;

  /*
  * callback after the measurement is stopped
  */
  void on_stop() override;

private:
  // channels used for this measurement
  const std::vector<channel_t> channels;

  // data variable
  std::vector<long long> customData;
};
