# -*- coding: utf-8 -*-
"""
Created on Thu Oct 31 12:08:32 2019

@author: Igor Shavrin

Product: Time Tagger
Swabian Instruments
https://www.swabianinstruments.com

Example on using Histogram2D measurement.
"""
import numpy as np
import matplotlib.pyplot as plt
from TimeTagger import createTimeTagger, freeTimeTagger, Histogram2D, Histogram, Correlation

tt = createTimeTagger()
tt.reset()  # Reset all settings to default values

# Channel definitions
START_CH = 1
INPUT_T1 = 2
INPUT_T2 = 3

# Histogram parameters
binwidth_1 = 1
binwidth_2 = binwidth_1
n_bins_1 = 400
n_bins_2 = 300

# Enable test signal
tt.setTestSignal([START_CH, INPUT_T1, INPUT_T2], True)

# Use StartStop measurement for both inputs to measure signal delays relative to start
corr1 = Correlation(tt, INPUT_T1, START_CH, 10, 10000)
corr2 = Correlation(tt, INPUT_T2, START_CH, 10, 10000)

plt.pause(2)

x1 = corr1.getIndex()
sd1 = corr1.getData()
idx1 = sd1.argmax()
delay1 = x1[idx1]

print('Measured delay at INPUT_T1: {:0.0f} ps'.format(delay1))

x2 = corr2.getIndex()
sd2 = corr2.getData()
idx2 = sd2.argmax()
delay2 = x2[idx2]

print('Measured delay at INPUT_T2: {:0.0f} ps'.format(delay2))

# Delays signals such that T1 and T2 by half of the histogram span
T1_delay = -delay1 + round(binwidth_1*n_bins_1/2)
T2_delay = -delay2 + round(binwidth_2*n_bins_2/2)

tt.setInputDelay(INPUT_T1, int(T1_delay))
tt.setInputDelay(INPUT_T2, int(T2_delay))

# Create 2D Histogram measurement
h2d = Histogram2D(tt,
                  start_channel=START_CH,
                  stop_channel_1=INPUT_T1,
                  stop_channel_2=INPUT_T2,
                  binwidth_1=binwidth_1,
                  binwidth_2=binwidth_2,
                  n_bins_1=n_bins_1,
                  n_bins_2=n_bins_2
                  )

# Wait for data to accumulate
plt.pause(2)

# Get and plot Histogram2D data
data2d = h2d.getData()
time1 = h2d.getIndex_1()
time2 = h2d.getIndex_2()

fig, ax = plt.subplots()

ax.pcolormesh(time1, time2, data2d.transpose())
ax.set_title('Histogram 2D')
ax.set_xlabel('T1 (ps)')
ax.set_ylabel('T2 (ps)')

freeTimeTagger(tt)

plt.show()
